<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Model\IterableApi;
use App\Helper\ResponseHelper;
use Log;
use App\Helper\Helper;

class IterableController extends Controller
{
    public $iterableObj;
    public $helper;
    private $helperObj;


    public function __construct()
    {
        $this->iterableObj = new IterableApi();
        $this->helper = new ResponseHelper();
        $this->helperObj = new Helper();
        
        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }
    
    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');
        if (!is_dir($this->logPath)){

         //Directory does not exist, so lets create it.
         mkdir($this->logPath);
         chmod($this->logPath, 0777);

        }
        $this->logFiles = array(
            'log' => array(
                "type" => "text",
                "filename" => $this->logPath . "/iterable-log.txt",
                "mode" => ""),
            );
    }
    
    public function setWithLogFiles() {

        if(!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                   $handler = fopen($fileName, $fileDetails['mode']);
                   fputcsv($handler, $fileDetails['columns']);
                   fclose($handler);
                   touch($fileName);
                   chmod($fileName, 0777);
                }
            }

        }
    }

    public function updateCustomer(Request $request) {
        
        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();

        }
        $this->helperObj->logMe("Reached to customer update. =>".$_REQUEST['data'], $this->logFiles['log']['filename']);
        $requestData = json_decode($_REQUEST['data']);

        $email = '';
        $quizAnswers = [];
        
        if(isset($requestData->email)){
            $email = $requestData->email;
        }
        
        if(isset($requestData->quizAnswer)){
            $quizAnswers = $requestData->quizAnswer;
        }
        

        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );
            $this->helperObj->logMe($response["errorMessage"], $this->logFiles['log']['filename']);
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();
        }

        if (empty($quizAnswers)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing quizAnswer input",
            );
            $this->helperObj->logMe($response["errorMessage"], $this->logFiles['log']['filename']);
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();
        }
        

        $assessment = '';
        $quizAnswerObj = [];
        $recoObject = [];
        $recommendationSkuObj = [];
        $shortFallCategories = [];
        $needStateCategories = [];
        
        
        
        foreach ($quizAnswers as $key => $value) {
            if(!empty($value)){
              $quizAnswerObj[$key] = trim($value);  
            }
        }
        
        if(isset($requestData->recommendation) && !empty($requestData->recommendation)){
            foreach ($requestData->recommendation as $rec) {
                if(!empty($rec)){
                  $recoObject[] = trim($rec);  
                }
            }
        }
        
        if(isset($requestData->shortFallCategories) && !empty($requestData->shortFallCategories)){
            foreach ($requestData->shortFallCategories as $shortFall) {
                if(!empty($shortFall)){
                  $shortFallCategories[] = trim($shortFall);  
                }
            }
        }
        
        if(isset($requestData->needStateCategories) && !empty($requestData->needStateCategories)){
            foreach ($requestData->needStateCategories as $specificNeed) {
                if(!empty($specificNeed)){
                  $needStateCategories[] = trim($specificNeed);  
                }
            }
        }
        
        if(isset($requestData->recommendationSKU) && !empty($requestData->recommendationSKU)){
            foreach ($requestData->recommendationSKU as $recSku) {
                if(!empty($recSku)){
                  $recommendationSkuObj[] = trim($recSku);
                }
            }
        }
        
        
        $userIterableUpdateRequest = [];
        
        if(!empty($email)){
            
            $userIterableUpdateRequest["email"] = $email;
           
            if(isset($requestData->name) && !empty($requestData->name)){
                $userIterableUpdateRequest['dataFields']["firstName"] = $requestData->name;
            }
            if(isset($requestData->acceptsMarketing) && !empty($requestData->acceptsMarketing)){
                $acceptsMarketing = false;
                if($requestData->acceptsMarketing == 'true'){
                    $acceptsMarketing = true;
                }
                $userIterableUpdateRequest['dataFields']["accepts_marketing"] = $acceptsMarketing;
            }
            /*if(isset($requestData->form) && !empty($requestData->form)){
                $userIterableUpdateRequest['dataFields']["form"] = $requestData->form;
            }*/
            if(!empty($quizAnswerObj)){
                $userIterableUpdateRequest['dataFields']['quizAnswer'] = $quizAnswerObj;
            }
            if(!empty($recoObject)){
                $userIterableUpdateRequest['dataFields']['recommendation'] = $recoObject;
            } 
            if(!empty($shortFallCategories)){
                $userIterableUpdateRequest['dataFields']['shortFallCategories'] = $shortFallCategories;
            } 
            if(!empty($needStateCategories)){
                $userIterableUpdateRequest['dataFields']['needStateCategories'] = $needStateCategories;
            }
            if(!empty($recommendationSkuObj)){
                $userIterableUpdateRequest['dataFields']['recommendationSKU'] = $recommendationSkuObj;
            }
            
            
            // echo "<pre>";
            // print_r($userIterableUpdateRequest);
            // die;
            $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
            
            
            /* If quiz Answer not empty event will generated*/
            if(!empty($quizAnswerObj)){
               $eventRequest = array(
                    "events" => array(
                        array(
                            "email" => strtolower($email),
                            "eventName" => "quizCompletion"
                        )
                    )
                );
    
                $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
        
                if (isset($eventResponse->successCount) && $eventResponse->successCount > 0) {
                    $this->helperObj->logMe("Quiz Completion event - Successfully created event for Email (".
                    strtolower($email).")", $this->logFiles['log']['filename']);
        
                } else {
                    $this->helperObj->logMe("Quiz Completion event - Failed to create event for Email (".
                    strtolower($email).")", $this->logFiles['log']['filename']);
        
                } 
            }
            
            

            if (isset($userIterableUpdateResponse->code) && strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
                
                $this->helperObj->logMe("Successfully updated the customer properties - ".$email.' => '.json_encode($userIterableUpdateRequest), $this->logFiles['log']['filename']);
                
                $response = array(
                    "status" => "success",
                    "message" => "Successfully updated the customer properties - ".$email,
                    "errorMessage" => "",
                );
        
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                exit();
            } else {
                
                $this->helperObj->logMe("Failed to update the customer properties - ".$email.' => '.json_encode($userIterableUpdateRequest), $this->logFiles['log']['filename']);
                $response         = array(
                    "status" => "failed",
                    "message" => "Failed to update the customer properties - ".$email,
                    "errorMessage"  => json_encode($userIterableUpdateResponse),
                );
        
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                exit();
            }
        }
        
    }

    public function quizCompletionEvent(Request $request) {
        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();

        }

        $requestData = json_decode($_REQUEST['data']);

        $email = $requestData->email;
        
        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();
        }

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => strtolower($email),
                    "eventName" => "quizCompletion"
                )
            )
        );

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
            $this->helperObj->logMe("Quiz Completion event - Successfully created event for Email (".
            strtolower($email).")", $this->logFiles['log']['filename']);
            

            $response = array(
                "status" => "success",
                "message" => "Quiz Completion event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();

        } else {
            $this->helperObj->logMe("Quiz Completion event - Failed to create event for Email (".
            strtolower($email).")", $this->logFiles['log']['filename']);
           

            $response = array(
                "status" => "success",
                "message" => "Quiz Completion event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();
        }

        
    }

    public function sendProspectEvent(Request $request)
    {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            if (empty($_REQUEST['callback'])) {
                echo json_encode($response);
            } else {
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            }
            exit();
        }

        $requestData = json_decode($_REQUEST['data']);

        $email = !empty($requestData->email) ? trim($requestData->email) : '';
        $is_hcp_pageStatus = !empty($requestData->isHcpPage) ? $requestData->isHcpPage : false;

        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();
        }
        
        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "accepts_marketing" => true
            )
        );
        
        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if (isset($userIterableUpdateResponse->code) && strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            $this->helperObj->logMe("Successfully updated the accept marketing properties for Email (".
            strtolower($email).")", $this->logFiles['log']['filename']);
        }else{
            
            $this->helperObj->logMe("Failed to update the accept marketing for Email (".
            strtolower($email).")", $this->logFiles['log']['filename']);
            
            $this->helperObj->logMe(json_encode($userIterableUpdateResponse), $this->logFiles['log']['filename']);
           
        }

        // Adding to the HCP List - Web Form list
        $listUpdateRequest = [];

        if ($is_hcp_pageStatus) {

            $listUpdateRequest = array(

                "listId" => intval(env('ITERABLE_HCP_LIST_ID', true)),
                "subscribers" => array(
                    array(
                        "email" => $email
                    )
                )
    
            );
                    
        } else {

            $listUpdateRequest = array(

                "listId" => intval(env('ITERABLE_DEFAULT_LIST_ID', true)),
                "subscribers" => array(
                    array(
                        "email" => $email
                    )
                )
    
            );
        }

        $createUserSubscribeListResponse = 
        $this->iterableObj->addSubscriberToIterableList(json_encode($listUpdateRequest));

        if (!empty($createUserSubscribeListResponse->msg) || 
        empty($createUserSubscribeListResponse)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($subscriptionResponse->msg)) {

                $error = json_encode($subscriptionResponse->msg);

            } else if (isset($subscriptionResponse->errors)) {

                $error = json_encode($subscriptionResponse->errors);

            }
            $this->helperObj->logMe("Prospect list update - Failed to update the list for Email (".
            strtolower($email).") Error:".$error,$this->logFiles['log']['filename']);
        }


        // Creating the event
        $eventRequest = array(
            "events" => array(
                array(
                    "email" => $email,
                    "eventName" => "prospect"
                )
            )
        );

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));
        

        if (isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
            
            $this->helperObj->logMe("Prospect event - Successfully created event for Email (".
            strtolower($email).")", $this->logFiles['log']['filename']);

            $response = array(
                "status" => "success",
                "message" => "Prospect event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();

        } else {
             $this->helperObj->logMe("Prospect event - Failed to create event for Email (".
            strtolower($email).")",$this->logFiles['log']['filename']);

            $response         = array(
                "status" => "failed",
                "message" => "Prospect event - Failed to create event - ".$email,
                "errorMessage"  => json_encode($eventResponse),
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            exit();
        }

    }

}
