<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Jobs\SendEditOrderMail;
use Log;


class OrderController extends Controller
{
    private $shopifyWebhookSecret;

    public function __construct() {

        $this->shopifyWebhookSecret = env('SHOPIFY_WEBHOOK_SECRET', true);
    }

    /**
     * Send order edit mail
     */
    public function sendOrderEditMail() {
       
        $hmacHeader = !empty($_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256']) ? trim($_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256']): '';
        $shopifyTopic = !empty($_SERVER['HTTP_X_SHOPIFY_TOPIC']) ? trim($_SERVER['HTTP_X_SHOPIFY_TOPIC']) : '';

        $data = file_get_contents('php://input');
        Log::channel('sendOrderEditMailLog')->info("Webhook topic:". $shopifyTopic);
        Log::channel('sendOrderEditMailLog')->info("Webhook request:". $data);

        $verified = $this->verify_shopify_webhook($data, $hmacHeader);
        $webhookStatus = var_export($verified, true);
        if ($verified) {

            $receivedData = json_decode($data);
            if (empty($receivedData->id)) {

                Log::channel('sendOrderEditMailLog')->info("Error in the webhook response(".$shopifyTopic.") - 
                Missing required parameters in the webhook response or the response is not correct");
            } else {

                $requestData = (object) array(
                    "data" => $data
                );
                
                $orderEditResponse = dispatch(new SendEditOrderMail($requestData, $shopifyTopic));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            Log::channel('sendOrderEditMailLog')->info("Order update webhook validation failed");
        }
    }

    /**
     * Verify Shopify webhook
     */
    public function verify_shopify_webhook($data, $hmac_header) {

        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $this->shopifyWebhookSecret, true));
        return hash_equals($hmac_header, $calculated_hmac);
    }
}
